@@ -266,24 +266,30 @@ struct op_absdiff
 template<>
 struct op_absdiff<schar, v_int8>
 {
+#if CV_SIMD
     static inline v_int8 r(const v_int8& a, const v_int8& b)
     { return v_absdiffs(a, b); }
+#endif
     static inline schar r(schar a, schar b)
     { return c_absdiff(a, b); }
 };
 template<>
 struct op_absdiff<short, v_int16>
 {
+#if CV_SIMD
     static inline v_int16 r(const v_int16& a, const v_int16& b)
     { return v_absdiffs(a, b); }
+#endif
     static inline short r(short a, short b)
     { return c_absdiff(a, b); }
 };
 template<>
 struct op_absdiff<int, v_int32>
 {
+#if CV_SIMD
     static inline v_int32 r(const v_int32& a, const v_int32& b)
     { return v_reinterpret_as_s32(v_absdiff(a, b)); }
+#endif
     static inline int r(int a, int b)
     { return c_absdiff(a, b); }
 };
@@ -1074,11 +1080,6 @@ struct scalar_loader_n<sizeof(float), OP, float, T2, v_float32>
         v_float32 r0 = op::r(v_src1, v_src2, scalar);
         v_float32 r1 = op::r(v_src1s, v_src2s, scalar);
 
-        #if CV_VERSION_MAJOR == 3
-        r0 = op::pre(v_src2, r0);
-        r1 = op::pre(v_src2s, r1);
-        #endif
-
         v_store(dst, r0);
         v_store(dst + step, r1);
     }
@@ -1091,11 +1092,6 @@ struct scalar_loader_n<sizeof(float), OP, float, T2, v_float32>
         v_float32 r0 = op::r(v_src1, scalar);
         v_float32 r1 = op::r(v_src1s, scalar);
 
-        #if CV_VERSION_MAJOR == 3
-        r0 = op::pre(v_src1, r0);
-        r1 = op::pre(v_src1s, r1);
-        #endif
-
         v_store(dst, r0);
         v_store(dst + step, r1);
     }
@@ -1184,11 +1180,6 @@ struct scalar_loader_n<sizeof(float), OP, float, double, v_float32>
         v_float32 r0 = r(v_src1, v_src2, scalar);
         v_float32 r1 = r(v_src1s, v_src2s, scalar);
 
-        #if CV_VERSION_MAJOR == 3
-        r0 = op::pre(v_src2, r0);
-        r1 = op::pre(v_src2s, r1);
-        #endif
-
         v_store(dst, r0);
         v_store(dst + step, r1);
     }
@@ -1200,11 +1191,6 @@ struct scalar_loader_n<sizeof(float), OP, float, double, v_float32>
         v_float32 r0 = r(v_src1, scalar);
         v_float32 r1 = r(v_src1s, scalar);
 
-        #if CV_VERSION_MAJOR == 3
-        r0 = op::pre(v_src1, r0);
-        r1 = op::pre(v_src1s, r1);
-        #endif
-
         v_store(dst, r0);
         v_store(dst + step, r1);
     }
@@ -1251,11 +1237,6 @@ struct scalar_loader_n<sizeof(double), OP, double, double, v_float64>
         v_float64 r0 = op::r(v_src1, v_src2, scalar);
         v_float64 r1 = op::r(v_src1s, v_src2s, scalar);
 
-        #if CV_VERSION_MAJOR == 3
-        r0 = op::pre(v_src2, r0);
-        r1 = op::pre(v_src2s, r1);
-        #endif
-
         v_store(dst, r0);
         v_store(dst + step, r1);
     }
@@ -1267,11 +1248,6 @@ struct scalar_loader_n<sizeof(double), OP, double, double, v_float64>
         v_float64 r0 = op::r(v_src1, scalar);
         v_float64 r1 = op::r(v_src1s, scalar);
 
-        #if CV_VERSION_MAJOR == 3
-        r0 = op::pre(v_src1, r0);
-        r1 = op::pre(v_src1s, r1);
-        #endif
-
         v_store(dst, r0);
         v_store(dst + step, r1);
     }
@@ -1460,11 +1436,13 @@ struct op_mul
 template<typename T1, typename T2, typename Tvec>
 struct op_mul_scale
 {
+#if CV_SIMD
     static inline v_float32 r(const v_float32& a, const v_float32& b, const T2* scalar)
     {
         const v_float32 v_scalar = vx_setall_f32(*scalar);
         return v_scalar * a * b;
     }
+#endif
     static inline T1 r(T1 a, T1 b, const T2* scalar)
     { return c_mul(a, b, *scalar); }
     static inline Tvec pre(const Tvec&, const Tvec& res)
@@ -1587,19 +1565,6 @@ DEFINE_SIMD_F64(mul, mul_loop_d)
 
 ///////////////////////////// Operations //////////////////////////////////
 
-#if CV_VERSION_MAJOR == 3
-template<typename T1, typename Tvec>
-struct op_div_f
-{
-    static inline Tvec r(const Tvec& a, const Tvec& b)
-    {
-        const Tvec v_zero = vx_setall<typename Tvec::lane_type>(0);
-        return v_select(b == v_zero, v_zero, a / b);
-    }
-    static inline T1 r(T1 a, T1 b)
-    { return b != (T1)0 ? a / b : (T1)0; }
-};
-#else
 template<typename T1, typename Tvec>
 struct op_div_f
 {
@@ -1608,11 +1573,11 @@ struct op_div_f
     static inline T1 r(T1 a, T1 b)
     { return a / b; }
 };
-#endif
 
 template<typename T1, typename T2, typename Tvec>
 struct op_div_scale
 {
+#if CV_SIMD
     static inline v_float32 r(const v_float32& a, const v_float32& b, const T2* scalar)
     {
         const v_float32 v_scalar = vx_setall_f32(*scalar);
@@ -1623,8 +1588,26 @@ struct op_div_scale
         const Tvec v_zero = vx_setall<typename Tvec::lane_type>(0);
         return v_select(denom == v_zero, v_zero, res);
     }
+#endif
     static inline T1 r(T1 a, T1 denom, const T2* scalar)
-    { return denom != (T1)0 ? c_div(a, denom, *scalar) : (T1)0; }
+    {
+        CV_StaticAssert(std::numeric_limits<T1>::is_integer, "");
+        return denom != (T1)0 ? c_div(a, denom, *scalar) : (T1)0;
+    }
+};
+
+template<>
+struct op_div_scale<float, float, v_float32>
+{
+#if CV_SIMD
+    static inline v_float32 r(const v_float32& a, const v_float32& b, const float* scalar)
+    {
+        const v_float32 v_scalar = vx_setall_f32(*scalar);
+        return a * v_scalar / b;
+    }
+#endif
+    static inline float r(float a, float denom, const float* scalar)
+    { return c_div(a, denom, *scalar); }
 };
 
 template<>
@@ -1636,14 +1619,9 @@ struct op_div_scale<double, double, v_float64>
         const v_float64 v_scalar = vx_setall_f64(*scalar);
         return a * v_scalar / b;
     }
-    static inline v_float64 pre(const v_float64& denom, const v_float64& res)
-    {
-        const v_float64 v_zero = vx_setzero_f64();
-        return v_select(denom == v_zero, v_zero, res);
-    }
 #endif
     static inline double r(double a, double denom, const double* scalar)
-    { return denom != 0.0 ? c_div(a, denom, *scalar) : 0.0; }
+    { return c_div(a, denom, *scalar); }
 };
 
 //////////////////////////// Loops /////////////////////////////////
@@ -1707,11 +1685,13 @@ DEFINE_SIMD_ALL(div, div_loop)
 template<typename T1, typename T2, typename Tvec>
 struct op_add_scale
 {
+#if CV_SIMD
     static inline v_float32 r(const v_float32& a, const v_float32& b, const T2* scalar)
     {
         const v_float32 v_alpha = vx_setall_f32(*scalar);
         return v_fma(a, v_alpha, b);
     }
+#endif
     static inline T1 r(T1 a, T1 b, const T2* scalar)
     { return c_add(a, b, *scalar); }
     static inline Tvec pre(const Tvec&, const Tvec& res)
@@ -1738,6 +1718,7 @@ struct op_add_scale<double, double, v_float64>
 template<typename T1, typename T2, typename Tvec>
 struct op_add_weighted
 {
+#if CV_SIMD
     static inline v_float32 r(const v_float32& a, const v_float32& b, const T2* scalars)
     {
         const v_float32 v_alpha = vx_setall_f32(scalars[0]);
@@ -1745,6 +1726,7 @@ struct op_add_weighted
         const v_float32 v_gamma = vx_setall_f32(scalars[2]);
         return v_fma(a, v_alpha, v_fma(b, v_beta, v_gamma));
     }
+#endif
     static inline T1 r(T1 a, T1 b, const T2* scalars)
     { return c_add(a, b, scalars[0], scalars[1], scalars[2]); }
     static inline Tvec pre(const Tvec&, const Tvec& res)
@@ -1853,6 +1835,7 @@ DEFINE_SIMD_F64(addWeighted, add_weighted_loop_d)
 template<typename T1, typename T2, typename Tvec>
 struct op_recip
 {
+#if CV_SIMD
     static inline v_float32 r(const v_float32& a, const T2* scalar)
     {
         const v_float32 v_scalar = vx_setall_f32(*scalar);
@@ -1863,8 +1846,26 @@ struct op_recip
         const Tvec v_zero = vx_setall<typename Tvec::lane_type>(0);
         return v_select(denom == v_zero, v_zero, res);
     }
+#endif
     static inline T1 r(T1 denom, const T2* scalar)
-    { return denom != (T1)0 ? c_div(*scalar, denom) : (T1)0; }
+    {
+        CV_StaticAssert(std::numeric_limits<T1>::is_integer, "");
+        return denom != (T1)0 ? c_div(*scalar, denom) : (T1)0;
+    }
+};
+
+template<>
+struct op_recip<float, float, v_float32>
+{
+#if CV_SIMD
+    static inline v_float32 r(const v_float32& a, const float* scalar)
+    {
+        const v_float32 v_scalar = vx_setall_f32(*scalar);
+        return v_scalar / a;
+    }
+#endif
+    static inline float r(float denom, const float* scalar)
+    { return c_div(*scalar, denom); }
 };
 
 template<>
@@ -1876,14 +1877,9 @@ struct op_recip<double, double, v_float64>
         const v_float64 v_scalar = vx_setall_f64(*scalar);
         return v_scalar / a;
     }
-    static inline v_float64 pre(const v_float64& denom, const v_float64& res)
-    {
-        const v_float64 v_zero = vx_setzero_f64();
-        return v_select(denom == v_zero, v_zero, res);
-    }
 #endif
     static inline double r(double denom, const double* scalar)
-    { return denom != 0.0 ? c_div(*scalar, denom) : 0.0; }
+    { return c_div(*scalar, denom); }
 };
 
 //////////////////////////// Loops /////////////////////////////////
@@ -1934,4 +1930,4 @@ DEFINE_SIMD_ALL(recip, recip_loop)
     #define SIMD_GUARD
 #endif
 
-}} // cv::hal::
\ No newline at end of file
+}} // cv::hal::
